/*
  pins_arduino.h - Pin definition functions for Arduino
  Part of Arduino - http://www.arduino.cc/

  Copyright (c) 2007 David A. Mellis

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA

  $Id: wiring.h 249 2007-02-03 16:52:51Z mellis $
*/

#ifndef Pins_Arduino_h
#define Pins_Arduino_h

#include <avr/pgmspace.h>

#define ARDUINO_MODEL_USB_PID	0x0034
#define FREEDUINO_MODEL_USB_PID 0xfc13
#define FREEDUINO_MODEL_USB_VID	0x190d

//#define TX_RX_LED_INIT	DDRD |= (1<<5), DDRB |= (1<<0)
#define TX_RX_LED_INIT
//#define TXLED0			PORTD |= (1<<5)
//#define TXLED1			PORTD &= ~(1<<5)
#define TXLED0
#define TXLED1
//#define RXLED0			PORTB |= (1<<0)
//#define RXLED1			PORTB &= ~(1<<0)
#define RXLED0
#define RXLED1

// Map SPI port to 'new' pins D14..D17
// D14				PB0					RXLED,SS/PCINT0
// D15				PB1					SCK,PCINT1
// D16				PB2					MOSI,PCINT2
// D17				PB3					MISO,PCINT3
const static uint8_t SS   = 14;
const static uint8_t MOSI = 16;
const static uint8_t MISO = 17;
const static uint8_t SCK  = 15;

const static uint8_t SDA = 12;
const static uint8_t SCL = 18;

const static uint8_t A0 = 19;
const static uint8_t A1 = 20;
const static uint8_t A2 = 21;
const static uint8_t A3 = 22;
const static uint8_t A4 = 23;
const static uint8_t A5 = 24;
const static uint8_t A6 = 25;  // 8;
const static uint8_t A7 = 26;  // 7;
const static uint8_t A8 = 27;  // 5;
const static uint8_t A9 = 28;  // 4;
const static uint8_t A10 = 29; // 2;
const static uint8_t A11 = 30; // new!

//	__AVR_ATmega32U4__ has an unusual mapping of pins to channels
extern const uint8_t PROGMEM analog_pin_to_channel_PGM[];
#define analogPinToChannel(P)  ( pgm_read_byte( analog_pin_to_channel_PGM + (P) ) )

#ifdef ARDUINO_MAIN

// On the Arduino board, digital pins are also used
// for the analog output (software PWM).  Analog input
// pins are a separate set.

// ATMEL ATMEGA32U4 / ARDUINO LEONARDO (by rel 1.0)
//
// D0				PD2					RXD1/INT2
// D1				PD3					TXD1/INT3
// D2				PD1		SDA			SDA/INT1
// D3#				PD0		PWM8/SCL	OC0B/SCL/INT0
// D4		A6		PD4					ADC8
// D5#				PC6		???			OC3A/#OC4A
// D6#		A7		PD7		FastPWM		#OC4D/ADC10
// D7				PE6					INT6/AIN0
//
// D8		A8		PB4					ADC11/PCINT4
// D9#		A9		PB5		PWM16		OC1A/#OC4B/ADC12/PCINT5
// D10#		A10		PB6		PWM16		OC1B/0c4B/ADC13/PCINT6
// D11#				PB7		PWM8/16		0C0A/OC1C/#RTS/PCINT7
// D12		A11		PD6					T1/#OC4D/ADC9
// D13#				PC7		PWM10		CLK0/OC4A
//
// A0		D18		PF7					ADC7
// A1		D19		PF6					ADC6
// A2		D20 	PF5					ADC5
// A3		D21 	PF4					ADC4
// A4		D22		PF1					ADC1
// A5		D23 	PF0					ADC0
//
// New pins D14..D17 to map SPI port to digital pins
//
// MISO		D14		PB3					MISO,PCINT3
// SCK		D15		PB1					SCK,PCINT1
// MOSI		D16		PB2					MOSI,PCINT2
// SS		D17		PB0					RXLED,SS/PCINT0
//
// TXLED			PD5
// RXLED		    PB0
// HWB				PE2					HWB

// FREEDUINO 32U4
// D13		PE6
// D14		PB0
// D15		PB1
// D16		PB2
// D17		PB3
// D18#		PD0
// D12		PD1
// D0		PD2
// D1		PD3
// D2		PD4   A
// D3		PD5
// D4		PD6   A
// D5#		PD7   A

// A0		PF0   A
// A1		PF1   A
// A2		PF4   A
// A3		PF5   A
// A4		PF6   A
// A5		PF7   A
// D11#		PC7
// D10#		PC6
// D9#		PB7
// D8#		PB6   A
// D7#		PB5   A
// D6		PB4   A (new)
		
// these arrays map port names (e.g. port B) to the
// appropriate addresses for various functions (e.g. reading
// and writing)
const uint16_t PROGMEM port_to_mode_PGM[] = {
	NOT_A_PORT,
	NOT_A_PORT,
	(uint16_t) &DDRB,
	(uint16_t) &DDRC,
	(uint16_t) &DDRD,
	(uint16_t) &DDRE,
	(uint16_t) &DDRF,
};

const uint16_t PROGMEM port_to_output_PGM[] = {
	NOT_A_PORT,
	NOT_A_PORT,
	(uint16_t) &PORTB,
	(uint16_t) &PORTC,
	(uint16_t) &PORTD,
	(uint16_t) &PORTE,
	(uint16_t) &PORTF,
};

const uint16_t PROGMEM port_to_input_PGM[] = {
	NOT_A_PORT,
	NOT_A_PORT,
	(uint16_t) &PINB,
	(uint16_t) &PINC,
	(uint16_t) &PIND,
	(uint16_t) &PINE,
	(uint16_t) &PINF,
};

const uint8_t PROGMEM digital_pin_to_port_PGM[31] = {
	PD, /* D0 = PD2 */
	PD, /* D1 = PD3 */
	PD, /* D2 = PD4 */
	PD, /* D3 = PD5 */
	PD, /* D4 = PD6 */
        PD, /* D5 = PD7 */
        
        PB, /* D6 = PB4 */
        PB, /* D7 = PB5 */
	PB, /* D8 = PB6 */
	PB, /* D9 = PB7 */
	PC, /* D10 = PC6 */
	PC, /* D11 = PC7 */

	PD, /* D12 = PD1 */
	
	PE, /* D13 = PE6 */
	PB,	/* 14 */
	PB,	/* 15 */
	PB,	/* 16 */
	PB,	/* 17 */
        
        PD, /* D18 = PD0 */

        PF, /* D19..D24 = A0..A5 */
        PF,
        PF,
        PF,
        PF,
        PF,

        PB, /* D25 = A6 = PB6 */  
	PB, /* D26 = A7 = PB5 */
        PD, /* D27 = A8 = PD7 */  
	PD, /* D28 = A9 = PD6 */
	PD, /* D29 = A10 = PD4 */
	PB, /* D29 = A11 = PB4 */

};

const uint8_t PROGMEM digital_pin_to_bit_mask_PGM[31] = {
	_BV(2), /* D0 = PD2 */
	_BV(3), /* D1 = PD3 */
	_BV(4), /* D2 = PD4 */
	_BV(5), /* D3 = PD5 */
	_BV(6), /* D4 = PD6 */
	_BV(7), /* D5 = PD7 */

	_BV(4),	/* D6 = PB4 */
	_BV(5), /* D7 = PB5 */
	_BV(6), /* D8 = PB6 */
	_BV(7), /* D9 = PB7 */
	_BV(6), /* D10 = PC6 */
	_BV(7), /* D11 = PC7 */
	
	_BV(1),	/* D12 = PD1 */

	_BV(6), /* D13 = PE6 */
	_BV(0), /* 14, port B */
	_BV(1),
	_BV(2),
	_BV(3),

        _BV(0), /* D18 = PD0 */

        _BV(0), /* D19..D24 = A0..A6 */
        _BV(1),
        _BV(4),
        _BV(5),
        _BV(6),
        _BV(7),

        _BV(6), /* D25 = A6 = PB6 */  
	_BV(5), /* D26 = A7 = PB5 */
        _BV(7), /* D27 = A8 = PD7 */  
	_BV(6), /* D28 = A9 = PD6 */
	_BV(4), /* D29 = A10 = PD4 */
	_BV(4), /* D29 = A11 = PB4 */

};

const uint8_t PROGMEM digital_pin_to_timer_PGM[31] = {
	NOT_ON_TIMER,	
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	TIMER4D,                /* 6 --> 5 */		
	NOT_ON_TIMER, 		
	TIMER1A,	        /* 9 --> 7 */
	TIMER1B,	        /* 10 --> 8 */
	TIMER0A,		/* 11 -> 9 */
	TIMER3A,		/* 5 --> 10 */
	TIMER4A,		/* 13 --> 11 */
	
	NOT_ON_TIMER,	
	NOT_ON_TIMER,		
	
	NOT_ON_TIMER,	
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,

        TIMER0B,                /* 3 --> 18 */
        NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,

        NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
};

const uint8_t PROGMEM analog_pin_to_channel_PGM[12] = {
	/*
         A0		PF0		ADC0
	 A1		PF1		ADC1
	 A2		PF4		ADC4
	 A3		PF5		ADC5
	 A4		PF6		ADC6
	 A5		PF7		ADC7
         D8/A6		PB6		ADC13  PWM16
         D9/A7		PB5		ADC12  PWM16    
 
         D5/A8		PD7		ADC10  FastPWM	
         D4/A9		PD6		ADC9		
	 D2/A10		PD4		ADC8  
         D6/A11		PB4		ADC11
	 */
	
	0,
	1,
	4,
	5,
	6,
	7,
        13,
        12,

	10,
	9,
	8,
	11
};

#endif

#endif
